"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ow_1 = require("ow");
const Hook_1 = require("../Hook");
const Test_1 = require("../Test");
const Emitter_1 = require("../Emitter");
const Contracts_1 = require("../Contracts");
class Group {
    constructor(_title, _resolveTestFn, _resolveHookFn, _options) {
        this._title = _title;
        this._resolveTestFn = _resolveTestFn;
        this._resolveHookFn = _resolveHookFn;
        this._options = _options;
        this._hooks = {
            before: [],
            after: [],
            beforeEach: [],
            afterEach: [],
        };
        this._error = null;
        this._completed = false;
        this._tests = [];
        this._hasFailingTests = false;
    }
    get hasErrors() {
        return this._hasFailingTests || !!this._error;
    }
    async _runHook(fn) {
        try {
            await fn.run();
        }
        catch (error) {
            this._completed = true;
            this._error = error;
        }
    }
    async _runTest(test) {
        for (let hook of this._hooks.beforeEach) {
            if (this._completed) {
                break;
            }
            await this._runHook(hook);
        }
        if (this._completed) {
            return;
        }
        await test.run();
        const testFailed = test.toJSON().status === Contracts_1.ITestStatus.FAILED;
        if (!this._hasFailingTests && testFailed) {
            this._hasFailingTests = true;
        }
        if (this._options.bail && testFailed) {
            this._completed = true;
            return;
        }
        for (let hook of this._hooks.afterEach) {
            if (this._completed) {
                break;
            }
            await this._runHook(hook);
        }
    }
    async _runTests() {
        for (let test of this._tests) {
            if (this._completed) {
                break;
            }
            if (!this._options.grep || this._options.grep.test(test.toJSON().title)) {
                await this._runTest(test);
            }
        }
    }
    toJSON() {
        let status = Contracts_1.IGroupStatus.PENDING;
        if (this._completed && this._error) {
            status = Contracts_1.IGroupStatus.FAILED;
        }
        else if (this._completed) {
            status = Contracts_1.IGroupStatus.PASSED;
        }
        return {
            title: this._title,
            status: status,
            error: this._error,
        };
    }
    timeout(duration) {
        ow_1.default(duration, ow_1.default.number.label('duration').integer);
        if (this._tests.length) {
            throw new Error('group.timeout must be called before defining the tests');
        }
        this._timeout = duration;
        return this;
    }
    test(title, callback, testOptions) {
        ow_1.default(title, ow_1.default.string.label('title').nonEmpty);
        testOptions = Object.assign({
            regression: false,
            skip: false,
            skipInCI: false,
            runInCI: false,
        }, testOptions);
        testOptions.timeout = this._timeout !== undefined ? this._timeout : this._options.timeout;
        const test = new Test_1.Test(title, this._resolveTestFn, callback, testOptions);
        this._tests.push(test);
        return test;
    }
    before(cb) {
        ow_1.default(cb, ow_1.default.function.label('cb'));
        this._hooks.before.push(new Hook_1.Hook(this._resolveHookFn, cb, 'before'));
        return this;
    }
    after(cb) {
        ow_1.default(cb, ow_1.default.function.label('cb'));
        this._hooks.after.push(new Hook_1.Hook(this._resolveHookFn, cb, 'after'));
        return this;
    }
    beforeEach(cb) {
        ow_1.default(cb, ow_1.default.function.label('cb'));
        this._hooks.beforeEach.push(new Hook_1.Hook(this._resolveHookFn, cb, 'beforeEach'));
        return this;
    }
    afterEach(cb) {
        ow_1.default(cb, ow_1.default.function.label('cb'));
        this._hooks.afterEach.push(new Hook_1.Hook(this._resolveHookFn, cb, 'afterEach'));
        return this;
    }
    async run() {
        Emitter_1.emitter.emit(Contracts_1.IEvents.GROUPSTARTED, this.toJSON());
        for (let hook of this._hooks.before) {
            if (this._completed) {
                break;
            }
            await this._runHook(hook);
        }
        if (!this._completed) {
            await this._runTests();
        }
        for (let hook of this._hooks.after) {
            if (this._completed) {
                break;
            }
            await this._runHook(hook);
        }
        this._completed = true;
        Emitter_1.emitter.emit(Contracts_1.IEvents.GROUPCOMPLETED, this.toJSON());
    }
}
exports.Group = Group;
