"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ow_1 = require("ow");
const timeSpan = require("time-span");
const retry = require("retry");
const isCI = require("is-ci");
const Callable_1 = require("../Callable");
const Emitter_1 = require("../Emitter");
const Exceptions_1 = require("../Exceptions");
const utils_1 = require("../utils");
const Contracts_1 = require("../Contracts");
class Test {
    constructor(_title, _resolveFn, _callback, options) {
        this._title = _title;
        this._resolveFn = _resolveFn;
        this._callback = _callback;
        this._regressionMessage = '';
        this._retries = 0;
        this._duration = 0;
        this._error = null;
        this._completed = false;
        this._todo = typeof (this._callback) !== 'function';
        this._timeout = options.timeout;
        this._regression = options.regression;
        if (options.skip) {
            this._skip = true;
        }
        else if (options.skipInCI && isCI) {
            this._skip = true;
        }
        else if (options.runInCI && !isCI) {
            this._skip = true;
        }
    }
    get _isHardException() {
        return utils_1.isCoreException(this._error);
    }
    _runTest() {
        return new Promise((resolve, reject) => {
            const op = retry.operation({ retries: this._retries, factor: 1 });
            op.attempt(async () => {
                Callable_1.Callable(this._resolveFn, this._callback, this._timeout)
                    .then(resolve)
                    .catch((error) => {
                    if (op.retry(error)) {
                        return;
                    }
                    reject(op.mainError());
                });
            });
        });
    }
    toJSON() {
        let status = Contracts_1.ITestStatus.PENDING;
        if (this._todo) {
            status = Contracts_1.ITestStatus.TODO;
        }
        else if (this._skip) {
            status = Contracts_1.ITestStatus.SKIPPED;
        }
        else if (this._completed && this._error) {
            status = (this._regression && !this._isHardException) ? Contracts_1.ITestStatus.PASSED : Contracts_1.ITestStatus.FAILED;
        }
        else if (this._completed && !this._error) {
            status = Contracts_1.ITestStatus.PASSED;
        }
        return {
            title: this._title,
            status: status,
            regression: this._regression,
            regressionMessage: this._regressionMessage,
            duration: this._duration,
            error: this._error,
        };
    }
    retry(counts) {
        ow_1.default(counts, ow_1.default.number.label('counts').integer);
        this._retries = counts;
        return this;
    }
    timeout(duration) {
        ow_1.default(duration, ow_1.default.number.label('duration').integer);
        this._timeout = duration;
        return this;
    }
    async run() {
        Emitter_1.emitter.emit(Contracts_1.IEvents.TESTSTARTED, this.toJSON());
        const start = timeSpan();
        if (!this._todo && !this._skip) {
            try {
                await this._runTest();
                if (this._regression) {
                    throw new Exceptions_1.RegressionException('Expected regression test to fail');
                }
            }
            catch (error) {
                this._error = error;
                if (!this._isHardException && this._regression) {
                    this._regressionMessage = error.message;
                }
            }
        }
        this._duration = start.rounded();
        this._completed = true;
        Emitter_1.emitter.emit(Contracts_1.IEvents.TESTCOMPLETED, this.toJSON());
    }
}
exports.Test = Test;
